<?php
# interface to popular dbs

define ('NopDB_wp', 1); // WordPress (blog, cms)
define ('NopDB_mg', 2); // Magento (shop)
define ('NopDB_dr', 3); // Drupal (cms)
define ('NopDB_jo', 4); // joomla (cms)
define ('NopDB_t3', 5); // typo3 (cms)
define ('NopDB_ps', 6); // prestashop (shop)
define ('NopDB_zc', 7); // zen-cart (shop)
define ('NopDB_oc', 8); // OpenCart (shop)

if(!defined('CR')) define('CR', "\r");
if(!defined('LF')) define('LF', "\n");
if(!defined('CRLF')) define('CRLF', CR.LF);
if(!defined('crlf')) define('crlf', CRLF);

/* blank method-template
    case NopDB_wp: break;
    case NopDB_mg: break;
    case NopDB_dr: break;
    case NopDB_jo: break;
    case NopDB_t3: break;
    case NopDB_ps: break;
    case NopDB_zc: break;
    case NopDB_oc: break;
*/

#  require_once('nopro_lib/nopro_sqli.php');
// nopro_sqli.php must be included here

class NopDB {
  public
    $pop = 0,
    $version,
    $base,
    $conf = false,
    $dbhost,
    $dbname,
    $dbuser,
    $dbpass,
    $prefix,
    $salt = null,
    $admin = null
    ;

  private
    $dbo = null,
    $tbl = array();

  public function conf($type) {
    // return relative path to config-file
    switch($type) {
    case NopDB_wp: return 'wp-config.php';
    case NopDB_mg: return 'app/etc/local.xml';
    case NopDB_dr: return 'sites/default/settings.php';
    case NopDB_jo: return 'configuration.php';
    case NopDB_t3: return 'typo3conf/LocalConfiguration.php';
    case NopDB_ps: return 'config/settings.inc.php';
    case NopDB_zc: return 'includes/configure.php';
    case NopDB_oc: return 'upload/config.php';
    default: return false;
    }
  }
  
  function __destruct() { $this->close(); }
  function __construct($type, $basedir) {
    $conf = $this->conf($type);
    if($conf === false || !file_exists("$basedir/$conf")) die("configuration '$basedir/$conf' not found\n");
    $this->pop = $type;
    $this->base = $basedir;
    $this->conf = "$basedir/$conf";
    if(!file_exists($this->conf)) return print("$this->conf not found" .crlf);
    switch($this->pop) {
    case NopDB_wp:
      include($this->conf);
      $this->dbhost = DB_HOST;
      $this->dbname = DB_NAME;
      $this->dbuser = DB_USER;
      $this->dbpass = DB_PASSWORD;
      $this->prefix = $table_prefix;
      
      if(empty($wp_version))
	include(ABSPATH . WPINC . '/version.php');
      $this->version = $wp_version;
      
      break;

      
    case NopDB_mg:
      $xml = new SimpleXMLElement($this->conf, 0, true);
      #print_r($xml);
      #print_r($xml->global->resources);
	
      $this->dbhost = (string) $xml->global->resources->default_setup->connection->host;
      $this->dbname = (string) $xml->global->resources->default_setup->connection->dbname;
      $this->dbuser = (string) $xml->global->resources->default_setup->connection->username;
      $this->dbpass = (string) $xml->global->resources->default_setup->connection->password;
      $this->prefix = (string) $xml->global->resources->db->table_prefix;
      $this->salt   = (string) $xml->global->crypt->key;
      $this->admin  = (string) $xml->admin->routers->adminhtml->args->frontName;
      $xml = null;
      break;
      
    case NopDB_dr:
      include($this->conf);
      $db = $databases['default']['default'];
      $this->dbhost = $db['host'];
      $this->dbname = $db['database'];
      $this->dbuser = $db['username'];
      $this->dbpass = $db['password'];
      $this->prefix = $db['prefix'];
      $this->salt   = $drupal_hash_salt;
      break;
      
    case NopDB_jo:
      include($this->conf);
      $j = new Jconfig();
      $this->dbhost = $j->host;
      $this->dbname = $j->db;
      $this->dbuser = $j->user;
      $this->dbpass = $j->password;
      $this->prefix = $j->dbprefix;
      $j = null;
      break;

    case NopDB_t3:
      function t3r($f) { return include($f); }
      $t3 = t3r($this->conf);
      $this->dbhost = $t3['DB']['host'];
      $this->dbname = $t3['DB']['database'];
      $this->dbuser = $t3['DB']['username'];
      $this->dbpass = $t3['DB']['password'];
      $this->prefix = '';
      break;

    case NopDB_ps:
      include($this->conf);
      $this->dbhost = _DB_SERVER_;
      $this->dbname = _DB_NAME_;
      $this->dbuser = _DB_USER_;
      $this->dbpass = _DB_PASSWD_;
      $this->prefix = _DB_PREFIX_;
      break;

    case NopDB_zc:
      include($this->conf);
      $this->dbhost = DB_SERVER;
      $this->dbname = DB_DATABASE;
      $this->dbuser = DB_SERVER_USERNAME;
      $this->dbpass = DB_SERVER_PASSWORD;
      $this->prefix = DB_PREFIX;
      break;

    case NopDB_oc:
      include($this->conf);
      $this->dbhost = DB_HOSTNAME;
      $this->dbname = DB_DATABASE;
      $this->dbuser = DB_USERNAME;
      $this->dbpass = DB_PASSWORD;
      $this->prefix = DB_PREFIX;
      break;

    }
    $err = error_reporting();
    error_reporting($err & !E_WARNING);
    $this->connect();
    error_reporting($err);
  }

  public function connect() {
    // connect to db
    $this->dbo = new mysqli($this->dbhost, $this->dbuser, $this->dbpass, $this->dbname);
    if($this->dbo->connect_errno) {
      print("connect(): ".$this->dbo->connect_error .crlf);
      $this->dbo = null;
    }
  }

  public function close() {
    // connect db-connection
    if($this->dbo !== null) {
      $this->dbo->close();
      $this->dbo = null;
    }
  }

  public function tables($upd = false, $all = false) {
    // load tbl[] with table_names matching prefix
    if($upd) $this->tbl = array();
    if(!$this->tbl) {
      if(!$all && $this->prefix) {
	foreach(sqli_tables($this->dbo) as $tbl)
	  if(strpos($tbl, $this->prefix) === 0)
	    $this->tbl[] = $tbl;
      } else
	$this->tbl = sqli_tables($this->dbo);
    }
    return $this->tbl;
  }

  public function dbo() { return $this->dbo; }
  public function table($table) { if(in_array($table, $this->tables())) return $table; else if(1) return $this->prefix . $table; else return null; }
  private function sane($msg) { if($this->dbo->errno) die($msg . $this->dbo->error); }
  public function query($query, $verb=0) { if($verb) echo $query.LF; $r = $this->dbo->query($query); $this->sane('query(): '); return $r; }
  public function optimize($verb = false) {
    // cleanup mysql-tables
    foreach($this->tables() as $tbl) {
      $q = "OPTIMIZE TABLE $tbl";
      if($verb)
	echo "$q\n";
      $this->query($q);
    }
  }
  
  public function boot() {
    // loads boot-strap
    switch($this->pop) {
    case NopDB_wp:
      #require_once($this->base . '/wp-load.php');
      include($this->base . '/wp-load.php');
      break;
      
    case NopDB_mg: 
    case NopDB_dr: 
    case NopDB_jo: 
    case NopDB_t3: 
    case NopDB_ps: 
    case NopDB_zc: 
    case NopDB_oc: 
    default: return false;
    }
  }
  
  public function addadmin($unam = 'nopro', $upass='sesame') {
    // add new user with admin-privileges

    define('HOWDY', '$P$BUdQCA9twaF1DnjH/L974GytPf/6Lq0');	 // hashed password 'howdy' for WP
    // by phpass-framework works on TYPO3, Drupal and Joomla also
    define('HOWDY_T3', '$1$vBp3Cbvf$9i0Ga0ySdLQai5HK4Qbxa/'); 	 // hashed password 'howdy' for TYPO3
    
    switch($this->pop) {
    case NopDB_wp:
      $tbl1 = $this->table('users'); 
      $tbl2 = $this->table('usermeta'); 

      $f1 = 'user_login';
      $q = "SELECT * FROM $tbl1 WHERE $f1 = '$unam' LIMIT 1";
      if($this->query($q)->num_rows > 0)
	return;

      $nupass = HOWDY;
      $q = "INSERT INTO $tbl1 SET `user_login`='$unam', user_pass='$nupass'"; echo "$q\n";
      $this->query($q);

      $uid = $this->qv("SELECT MAX(ID) FROM $tbl1");
      // copy all meta from user 1 to new uid
      $q = "SELECT meta_key, meta_value FROM $tbl2 WHERE user_id=1"; 
      $res = $this->query($q);
      while($row = $res->fetch_assoc()) {
	$k = $row['meta_key'];
	$v = $row['meta_value'];
	$q = "INSERT INTO $tbl2 SET user_id=$uid, `meta_key`='$k', `meta_value`='$v'"; #echo "$q\n";
	$this->query($q);
      }
      $res->close();

      $q = "UPDATE $tbl2 SET meta_value='' WHERE user_id=$uid AND meta_key like '%_name'"; echo "$q\n";
      $this->query($q);
      $q = "UPDATE $tbl2 SET meta_value='$unam' WHERE user_id=$uid AND meta_key='nickname'"; echo "$q\n";
      $this->query($q);
      break;

    case NopDB_mg:
      $tbl1 = $this->table('admin_user'); 
      $tbl2 = $this->table('admin_role');

      $f1 = 'username';
      $q = "SELECT * FROM $tbl1 WHERE $f1 = '$unam' LIMIT 1";
      if($this->query($q)->num_rows > 0) {
	echo "User $unam already exists" .LF;
	return;
      }

      // ancient versions had only varchar(40) as type for password-field
      $passlen = 0;
      foreach(sqli2arr($this->dbo, 'explain ' . $tbl1) as $xpl) {
	if($xpl['Field'] == "password") {
	  $passlen = preg_replace('/^[^0-9]*([0-9]*)[^0-9]*$/', '$1', $xpl['Type']);
	  if($passlen == 0) $passlen = 100;
	  break;
	}
      }
      $salt0 = substr($this->salt, 0, $passlen - 34); // md5 + ':' as ASCIIZ
      //die("passlen $passlen\nsalt  ".$this->salt."\nsalt0 $salt0");

      $entry = array(
		     #'firstname' => 'NoPro',
		     #'lastname'  => 'NoPro',
		     'username'  => $unam,
		     'password'  => '',
		     'email'     => 'nopro4biz@gmail.com',
		     #'extra'     => 'N;'
		     );
      sqlistore($this->dbo, $tbl1, $entry);
      $q = "SELECT user_id FROM $tbl1 ORDER BY user_id DESC LIMIT 1";
      $uid = $this->qv($q);
      $q = "UPDATE $tbl1 SET password = CONCAT(MD5('$salt0$upass'), ':$salt0') WHERE user_id = '$uid'";
      #$q = "UPDATE $tbl1 SET password = CONCAT(MD5('$salt0$upass'), ':$salt0') WHERE username = '$unam'";
      echo $q .CRLF;
      $this->query($q);

      $q = "SELECT * FROM $tbl2 WHERE role_type = 'U' ORDER BY user_id LIMIT 1";
      $arr2 = $this->q2($q);
      
      array_shift($arr2);	// shift role_id
      $arr2['role_name'] = 'NoPro';
      $arr2['user_id'] = $uid;
      sqlistore($this->dbo, $tbl2, $arr2);
      break;
      
    case NopDB_dr:
      $tbl1 = $this->table('users'); 
      $tbl2 = $this->table('role'); 
      $tbl3 = $this->table('users_roles');

      $f1 = 'name';
      $q = "SELECT * FROM $tbl1 WHERE $f1 = '$unam' LIMIT 1";
      if($this->query($q)->num_rows > 0)
	return;
      
      $uid = $this->qv("SELECT MAX(UID) FROM $tbl1");
      $entry = array(
		     'uid'    => ++$uid,
		     'name'   => $unam,
		     'pass'   => HOWDY,
		     'mail'   => 'nopro4biz@gmail.com',
		     'status' => 1
		     );
      sqlistore($this->dbo, $tbl1, $entry);
      $gid = $this->qv("SELECT rid FROM $tbl2 WHERE name ='administrator'");
      $this->query("INSERT INTO $tbl3 VALUES ($uid, $gid)");
      break;

    case NopDB_jo:
      // insert user and an entry into user-group-map
      $tbl1 = $this->table('users'); 
      $tbl2 = $this->table('usergroups'); 
      $tbl3 = $this->table('user_usergroup_map');
      
      $f1 = 'username';
      $q = "SELECT * FROM $tbl1 WHERE $f1 = '$unam' LIMIT 1";
      if($this->query($q)->num_rows > 0)
	return;

      $entry = array(
		     'username'  => $unam,
		     'password'  => HOWDY,
		     'email'     => 'nopro4biz@gmail.com',
		     );
      sqlistore($this->dbo, $tbl1, $entry);
      $uid = $this->qv("SELECT MAX(id) FROM $tbl1");
      $gid = $this->qv("SELECT id FROM $tbl2 WHERE title ='Super Users'");
      $this->query("INSERT INTO $tbl3 VALUES ($uid, $gid)");
      break;
      
    case NopDB_t3:
      $tbl1 = $this->table('be_users'); 

      $f1 = 'username';
      $q = "SELECT * FROM $tbl1 WHERE $f1 = '$unam' LIMIT 1";
      if($this->query($q)->num_rows > 0)
	return;

      $entry = array(
		     'username'  => $unam,
		     'password'  => HOWDY,
		     'email'     => 'nopro4biz@gmail.com',
		     'admin'     => 1,
		     #'uc'        => '',
		     #'workspace_perms' => 1,
		     );
      sqlistore($this->dbo, $tbl1, $entry);
      break;
      
    case NopDB_ps: break;
    case NopDB_zc: break;
    case NopDB_oc: break;
    default: return false;
    }
  }
  
  public function q1($query) { $res = $this->query($query); $row = $res->fetch_row(); $res->close(); return $row; }
  public function q2($query) { $res = $this->query($query); $row = $res->fetch_assoc(); $res->close(); return $row; }
  public function qv($query) { return $this->q1($query)[0]; }
  
  public function table_status($table) {
    if($this->table($table) === null) return null;
    $q = sprintf("SHOW TABLE STATUS LIKE '%s'", $this->table($table));
    return $this->q2($q);
  }

  public function boundary($table, $field, $quotient) {
    // return value-limit for tbl
    if($this->table($table) === null) return array();
    $n = $this->qv("SELECT COUNT(DISTINCT $field) FROM $table");
    $l = floor($n * $quotient);
    return $this->qv("SELECT DISTINCT $field FROM $table ORDER BY $field LIMIT $l,1");
  }

  public function coll($query, $dim=1) {
      if($dim > 1)
          return sqli2arr($this->dbo, $query);
      else
          return sqli2arr1d($this->dbo, $query);
  }
  
}
?>
